<?php

/* 
 * * ******************************************************************
 *
 *   CREATED BY MODULESGARDEN       ->        http://modulesgarden.com
 *   AUTHOR                         ->     dariusz.bi@modulesgarden.com
 *   CONTACT                        ->       contact@modulesgarden.com
 *
 *  This software is furnished under a license and may be used and copied
 *  only  in  accordance  with  the  terms  of such  license and with the
 *  inclusion of the above copyright notice.  This software  or any other
 *  copies thereof may not be provided or otherwise made available to any
 *  other person.  No title to and  ownership of the  software is  hereby
 *  transferred.
 *
 * * ******************************************************************
 */

class CertumSSLMainController extends AbstractControler
{
    public $dbConfiguration = false;
    public $clientUsername = false;
    private $allowRawExeptionMessages = false;
    
    function __construct($type,$dir,$language  = 'english') 
    { 
        
        MGLang::getInstance($dir.DS.'lang'.DS,$language);
        try{
            $mainConfig = array(
                    'dir'               => $dir
                    ,'type'             => $type
            );
           
            $additionalConfig = array();
            
            if(file_exists($dir.DS.'additionalConfig.php')){
                include $dir.DS.'additionalConfig.php';
            }
            
            if(isset($additionalConfig['baseUrlConfig'])){
                $mainConfig['baseUrlConfig'] = $additionalConfig['baseUrlConfig'];
            }

            parent::__construct($mainConfig);
            
            $localAPIClass = $this->type.'LocalAPIDriver';
            
            if(!class_exists($localAPIClass)){
                die('Unable to find local API class:'.$localAPIClass);
            }

            CertumSSLDriver::localAPI($localAPIClass,$this);
            if($type == 'CPanel'){
                $this->dbConfiguration = CertumSSLDriver::localAPI()->getDatabaseConfiguration();
            }

            $this->prepare();

        }catch(Exception $ex){
            die($ex->getMessage());
        }
    }
      
    private function prepare() 
    {
       $configClass     = $this->type.'ConfigDriver';
       
       if(!class_exists($configClass)){
           die('Unable to find file for config driver:'.$configClass);
       }

       $dbConfig = new $configClass($this); 

       if(!$dbConfig->isNotEmpty()){
           die('Empty DB Configuration. Unable To Load DB Configuration');
       }

       MGMySQL::getInstance($dbConfig->db_host, $dbConfig->db_user, $dbConfig->db_pass, $dbConfig->db_name);

       CertumSSLDriver::config('CertumSSLConfiguration',$this);
       
       $remoteAPIClass = 'PartnerAPIService';
       
       if(!class_exists($remoteAPIClass)){
           die('Unable to find remote API class:'.$remoteAPIClass);
       }
            
       CertumSSLDriver::remoteAPI($remoteAPIClass,$this);

    }
    
    //Generate Custom HTML
    
    function getHTMLPage($page = 'Main', $action = 'Main', $input = array(), $customParams=array())
    {      
        $controlerConfig = $this->searchControlers();
        
        $page = ($page === null)?$controlerConfig['default']:$page;
        $action = ($action === null)?'Main':$action;

        $content = array(
            'pages'         => $controlerConfig['controllers']
            ,'currentPage'  => $page
            ,'currenAjax'   => $this->getAJAX($page)
            ,'content'      => null
            ,'error'        => null
            ,'customParams' => $customParams
        );
        
        try{
            $this->prepare();
            
            $data = $this->runAction($page, $action, 'HTML',$input);
            
            if(empty($data['template'])){
                die('Template not provided');
            }

            if($data){
                MGLang::stagCurrentContext('runAction');
                MGLang::setContext($page);
                $content['content'] = $this->getHTML($data['template'], $data['vars']);
                MGLang::unstagContext('runAction');
            }
        } catch(Exception $ex){
            CertumSSLExceptions::addLog($ex);
                        
            $message = null;
            
            if(method_exists($ex, 'getUserMessage')){
                $message = $ex->getUserMessage();
            }
                       
            if(empty($message)){
                $message = 'general';
            }

            $messageLang = MGLang::absoluteT($page,'errorMessages',$message);
            
            if($message == $messageLang){
                $messageLang = MGLang::absoluteT('errorMessages',$message);
            }
            
            if($message == $messageLang){
                $messageLang = MGLang::absoluteT('errorMessages','general');
            }
            
            if($this->allowRawExeptionMessages && $message == $messageLang){
                $messageLang = $ex->getMessage();
            }
            
            $content['error'] = $messageLang;
            
            if(method_exists($ex, 'getToken')){
                $content['errorID'] = $ex->getToken();
            }
        }
        
        try{
            MGLang::stagCurrentContext('createContainer');
            MGLang::setContext('Container');
            $html = $this->getHTML('Container',$content);
            MGLang::unstagContext('createContainer');
            return $html;
        } catch (Exception $ex) {
            CertumSSLExceptions::addLog($ex);
            
            $message = null;
            
            if(method_exists($ex, 'getUserMessage')){
                $message = $ex->getUserMessage();
            }
            
            if(empty($message)){
                $message = 'general';
            }
            
            MGLang::setContext('Main','errorMessages');
            $message = MGLang::T($message);
            
            if(method_exists($ex, 'getToken')){
                $message .= MGLang::absoluteT('errorMessages','errorID').$ex->getToken();
            }
            
            die($message);
        }
    }
    
    function searchControlers()
    {
        $controllers = array();
        
        $configFile = $this->dir.DS.'controllers'.DS.$this->type.'EnabledControllers.ini';
        
        if(!file_exists($configFile)){
            die('Unable to find controllers configuration file:'.$configFile);
        }
        
        $config = parse_ini_file($configFile);
 
        foreach($config['controller'] as $controller){
            $controllerFile = $this->dir.DS.'controllers'.DS.$this->type.$controller.'Controller.php';
            if(file_exists($controllerFile))
            {
                $controllers[] = $controller;
            }
        }

        return array(
            'controllers'   => $controllers
            ,'default'      => $config['defaultController']
        );
    }
    
    private function runAction($page = 'Main',$action = 'Main',$type='HTML',$input = array())
    {        
        $controllersDir     = $this->dir.DS.'controllers'.DS;
        $controllerClass    = $this->type.$page.'Controller';
        $controllerFile     = $controllersDir.$controllerClass.'.php';
                
        if(!file_exists($controllerFile)){
            die("Unable To Find File: ".$controllerFile);
        }

        require_once $controllerFile;

        if(!class_exists($controllerClass)){
            die("Unable To Find Class: ".$controllerClass.' Controller Not Exits');
        }
        
        $controllerMethod = $action.$type.'Action';
       
        if(!method_exists($controllerClass, $controllerMethod)){
            die("Unable To Find Method: ".$controllerMethod.' Action Not Exits');
        }
        
        MGLang::stagCurrentContext('runAction');
        MGLang::setContext($page);

        $controllerObject = new $controllerClass($this,array('page'=>$page));

        $data = $controllerObject->$controllerMethod($input);

        MGLang::unstagContext('runAction');
        
        return $data;
    }
    
    function runCron($controler, $action){
        try{
            $this->prepare();
            $this->runAction($controler, $action, 'Cron');
        } catch (Exception $ex) {
            return $ex->getMessage();
        }
    }
    
    public function setClientArea()
    {
        $this->clientUsername = CertumSSLDriver::localAPI()->getCurrentUser();
        MGLang::loadLang(CertumSSLDriver::localAPI()->getCurrentLang());
    }
    
    public function setAdminArea(){
        $this->allowRawExeptionMessages = true;
    }
    
    function getJSONResponse($page = 'Main', $action = 'Main', $input = array()){
                
        $controlerConfig = $this->searchControlers();
        
        $page = ($page === null)?$controlerConfig['default']:$page;
        $action = ($action === null)?'Main':$action;
        
        $content = array(
            'action'        => $page
            ,'result'       => null
        );
        
        try{
            $this->prepare();
            $content['result']  = 'success';
            $content['success'] = $this->runAction($page, $action, 'JSON', $input);
        } catch (Exception $ex) {                   
            $message = $ex->getMessage();
            $messageLang = MGLang::absoluteT($page,'errorMessages',$message);

            $content['result']  = 'error';
            $content['error'] = $messageLang;
        }
        
        return $this->getJSON($content);
    }
    
}