<?php
/*
 * * ******************************************************************
 *
 *   CREATED BY MODULESGARDEN       ->        http://modulesgarden.com
 *   AUTHOR                         ->     dariusz.bi@modulesgarden.com
 *   CONTACT                        ->       contact@modulesgarden.com
 *
 *  This software is furnished under a license and may be used and copied
 *  only  in  accordance  with  the  terms  of such  license and with the
 *  inclusion of the above copyright notice.  This software  or any other
 *  copies thereof may not be provided or otherwise made available to any
 *  other person.  No title to and  ownership of the  software is  hereby
 *  transferred.
 *
 * * ******************************************************************
 */

class MGCertOrders
{
    private $_tableName;
    private $relId;
    private $userId;
    private $domainId;
    private $orderId;
    private $csr;
    private $key;
    private $crt;
    private $approveCode;
    private $orderVoucher;
    private $activateDate;
    private $createDate;
    private $status;
    private $details;

    function __construct(array $params = array())
    {
        $this->load($params);

        if (empty($this->createDate)) {
            $this->createDate = date('Y-m-d H:i:s');
        }
    }

    function load(array $params)
    {
        foreach ($params as $propertyName => $propertyValue) {
            if (property_exists($this, $propertyName)) {
                $this->{$propertyName} = $propertyValue;
            }
        }
    }

    function create()
    {
        $toSave = clone $this;

        MGMySQL::insert($toSave->_tableName,
            array(
            'rel_id' => $toSave->relId,
            'order_id' => $toSave->orderId,
            'user_id' => $toSave->userId,
            'domain_id' => $toSave->domainId,
            'used_voucher' => $toSave->orderVoucher,
            'used_csr' => $toSave->csr,
            'csr_key' => $toSave->key,
            'crt' => $toSave->crt,
            'details' => $toSave->details,
            'file_name' => $toSave->approveCode,
            'file_content' => $toSave->approveCode,
            'status' => $toSave->status,
            'voucher_activate_date' => $toSave->activateDate,
            'date' => $toSave->createDate
        ));
    }

    function updateOrderID($oldOrderID)
    {
        $toSave = clone $this;

        MGMySQL::update($toSave->_tableName, array(
            'order_id' => $toSave->orderId,
            ), array(
            'order_id' => $oldOrderID,
            'user_id' => $toSave->userId,
            'domain_id' => $toSave->domainId
            )
        );
    }

    function updateStatus($status)
    {
        $toSave = clone $this;

        MGMySQL::update($toSave->_tableName, array(
            'status' => $status,
            ), array(
            'order_id' => $toSave->orderId
            )
        );
    }

    function setFinished($finished = 0)
    {
        $toSave = clone $this;

        MGMySQL::update($toSave->_tableName, array(
            'Finished' => $finished,
            ), array(
            'order_id' => $toSave->orderId
            )
        );
    }

    function updateFileCreated($orderID, $status = 0)
    {
        $toSave = clone $this;

        MGMySQL::update($toSave->_tableName, array(
            'file_created' => $status,
            ), array(
            'order_id' => $orderID
            )
        );
    }

    function getAllOrders()
    {
        $toSave = clone $this;

        $q = MGMySQL::query("
            SELECT 
                * 
            FROM 
                `".$toSave->_tableName."`
            WHERE 
                file_created = 0
            ORDER BY
                id
            ASC")->fetchAll();

        return $q;
    }

    function getAllNotFinishedOrders()
    {
        $toSave = clone $this;

        $q = MGMySQL::query("
            SELECT 
                * 
            FROM 
                `".$toSave->_tableName."`
            WHERE 
                Finished = 0
            ORDER BY
                id
            ASC")->fetchAll();

        $q2 = MGMySQL::query("
            SELECT 
                * 
            FROM 
                `certumssl_reissued_orders`
            WHERE 
                Finished = 0
            ORDER BY
                id
            ASC")->fetchAll();



        return array_merge($q, $q2);
    }

    function getOrderByVoucher($voucherCode)
    {
        $toSave = clone $this;

        $q = MGMySQL::query("
            SELECT
                *
            FROM
                `".$toSave->_tableName."`
            WHERE
                used_voucher = '".$voucherCode."'
            ORDER BY
                id
            ASC")->fetchAll();

        if ($q) {
            return true;
        }

        return false;
    }

    function getParentParams($parentID)
    {
        $toSave = clone $this;

        $q = MGMySQL::query("
            SELECT 
                user_id , domain_id, used_csr, csr_key, file_name
            FROM 
                `".$toSave->_tableName."`
            WHERE
                order_id = '".$parentID."'
        ")->fetchAll();

        return $q;
    }

    function getDetailsByOrderId($id)
    {
        $toSave = clone $this;

        $q = MGMySQL::query("
            SELECT 
                details
            FROM 
                `".$toSave->_tableName."`
            WHERE
                order_id = '".$id."'
        ")->fetch();

        return json_decode($q["details"], true);
    }

    function getUserOrders()
    {
        $toSave = clone $this;

        $q = MGMySQL::query("
            SELECT 
                *
            FROM
                `".$toSave->_tableName."`
            WHERE 
                user_id = '".$toSave->userId."'
            ORDER BY
                id ASC  
        ")->fetchAll();

        $ordersList = array();
        foreach ($q as $r) {
            $ordersList[] = array(
                'domainid' => $r['domain_id'],
                'relid' => $r['rel_id'],
                'token' => $r['used_voucher'],
                'userid' => $r['user_id'],
                'orderid' => $r['order_id'],
                'activateddate' => $r['voucher_activate_date'],
                'finished' => $r['Finished'],
                'status' => $r['status']
            );
        }

        return $ordersList;
    }
}