<?php

//
/*
 * * ******************************************************************
 *
 *   CREATED BY MODULESGARDEN       ->        http://modulesgarden.com
 *   AUTHOR                         ->     dariusz.bi@modulesgarden.com
 *   CONTACT                        ->       contact@modulesgarden.com
 *
 *  This software is furnished under a license and may be used and copied
 *  only  in  accordance  with  the  terms  of such  license and with the
 *  inclusion of the above copyright notice.  This software  or any other
 *  copies thereof may not be provided or otherwise made available to any
 *  other person.  No title to and  ownership of the  software is  hereby
 *  transferred.
 *
 * * ******************************************************************
 */

class CPanelLocalAPIDriver extends WHMLocalAPIDriver
{
    public $clientUsername;
    public $cpanel;

    function __construct($params)
    {
        $this->cpanel = $GLOBALS['CPANEL'];
        parent::__construct($params);
    }

    public function _connect($params)
    {
        $this->apiKey = preg_replace("'(\r|\n)'", "", $this->getAccessHash());

        if (empty($this->apiKey)) {
            throw new Exception("Cant find Acces Hash File", 404);
        }

        $this->clientUsername = $this->getCurrentUser();
    }

    private function _callUAPI($module, $action, $params = array())
    {
        $ret = $this->cpanel->uapi($module, $action, $params);

        if (!empty($ret['cpanelresult']['result']['errors'])) {
            die(implode('/', $ret['cpanelresult']['result']['errors']));
        }

        return $ret['cpanelresult']['result']['data'];
    }

    private function _callAPI1($module, $action, $params = array())
    {
        $ret = $this->cpanel->api1($module, $action, $params);

        if (!empty($ret['cpanelresult']['result']['errors'])) {
            die(implode('/', $ret['cpanelresult']['result']['errors']));
        }

        return $ret['cpanelresult']['data']['result'];
    }

    function getDomainCertyficate($domain)
    {

        $certyfiactes = $this->_callUAPI('SSL', 'installed_hosts');

        foreach ($certyfiactes as $certyfiacte) {
            foreach ($certyfiacte["domains"] as $certDomain) {
                if ($certDomain == $domain) {
                    return $certyfiacte['certificate_text'];
                }
            }
        }

        return false;
    }

    function getDatabaseConfiguration()
    {
        return $this->_callUAPI('CertumSSL', 'getConfiguration');
    }

    function getAccessHash()
    {
        return $this->_callUAPI('CertumSSL', 'getAccessHash');
    }

    function getCurrentUser()
    {
        return $this->cpanel->cpanelprint('$user');
    }

    function getCurrentLang()
    {
        return $this->cpanel->cpanelprint('$lang');
    }

    function getFooter()
    {
        if ($this->cpanel->cpanelprint('$theme') == 'paper_lantern') {
            return $this->cpanel->footer();
        } else {
            return $this->_callAPI1('Branding', 'include', array('stdfooter.html'));
        }
    }

    function getHeader()
    {
        if ($this->cpanel->cpanelprint('$theme') == 'paper_lantern') {
            return $this->cpanel->header("CertumSSL");
        } else {
            return $this->_callAPI1('Branding', 'include', array('stdheader.html'));
        }
    }

    function getDomainRelPatch($domain)
    {
        $result = $this->_userRequest($this->clientUsername, 'DomainLookup', 'getdocroot', array('domain' => $domain));
        return $result['cpanelresult']['data'][0]['reldocroot'];
    }

    function putValidationFile($domain, $filename, $content)
    {
        $domainPath = $this->getDomainRelPatch($domain);
        $this->_userRequest($this->clientUsername, 'Fileman', 'savefile', array('dir' => $domainPath, 'filename' => $filename, 'content' => $content));
        $result     = $this->_userRequest($this->clientUsername, 'Fileman', 'viewfile', array('dir' => $domainPath, 'file' => $filename));
        if (!(isset($result['cpanelresult']['data'][0]['contents']) && $result['cpanelresult']['data'][0]['contents'] == $content)) {
            die('API ERROR cpanelError Unable to upload csr hash file for the domain: '.$domain);
        }
    }

    function generateCsr($params = null)
    {
        $details = array(
            'domain' => $params['domain'],
            'country' => $params['country'],
            'state' => $params['state'],
            'city' => $params['city'],
            'OrganizationName' => $params['OrganizationName'],
            'organisation_unit' => $params['organisation_unit'],
        );

        $generateKeyResult = $this->_callUAPI('SSL', 'generate_key', array('keysize' => '2048'));

        $domains = "";
        if (is_array($params['domain'])) {
            $i = -1;
            foreach ($params['domain'] as $domain) {
                $i++;
                if ($i < count($params['domain'])) {
                    $domains .= $domain.',';
                } else {
                    $domains .= $domain;
                }
            }
        } else {
            $domains = $params['domain'];
        }

        $certParams = array(
            'key_id' => $generateKeyResult['id'],
            'domains' => $domains, //$params['domain'],
            'countryName' => $params['country'],
            'stateOrProvinceName' => mb_convert_encoding($params['state'], "UTF-8"),
            'localityName' => mb_convert_encoding($params['city'], "UTF-8"),
            'organizationName' => mb_convert_encoding($params['OrganizationName'], "UTF-8"),
            'emailAddress' => $params['email']
        );

        if (isset($params['organisation_unit'])) {
            $certParams['organizationalUnitName'] = $params['organisation_unit'];
        }


        $generateCrtResult = $this->_callUAPI('SSL', 'generate_cert', $certParams);

        $csrParams = array(
            'key_id' => $generateKeyResult['id'],
            'domains' => $domains, //$params['domain'],
            'countryName' => $params['country'],
            'stateOrProvinceName' => mb_convert_encoding($params['state'], "UTF-8"),
            'localityName' => mb_convert_encoding($params['city'], "UTF-8"),
            'organizationName' => mb_convert_encoding($params['OrganizationName'], "UTF-8"),
            'emailAddress' => $params['email']
        );
        if (isset($params['organisation_unit'])) {
            $csrParams['organizationalUnitName'] = $params['organisation_unit'];
        }
        $generateCsrResult = $this->_callUAPI('SSL', 'generate_csr', $csrParams);



        return array(
            'csr' => $generateCsrResult['text'],
            'key' => $generateKeyResult['text'],
            'crt' => $generateCrtResult['text'],
            'details' => $details
        );
    }

    function getUserDomains($user = null, $main = null)
    {
        return parent::getUserDomains($this->clientUsername, $main);
    }

    function getMainDomain($user = null)
    {
        return parent::getMainDomain($this->clientUsername);
    }

    function getUserCerts($user = null)
    {
        $output = $this->_callUAPI('SSL', 'list_certs');
        return $output;
    }

    function getUserCert($user = null, $certID = null)
    {
        $output = $this->_callUAPI('SSL', 'show_cert', array(
            'id' => $certID
        ));
        return $output;
    }

    function getAccounts()
    {
        return array();
    }
}