<?php
require '/usr/local/directadmin/plugins/certum_ssl/helper/CertumSSLHelper.php';

class DirectadminMainController extends AbstractControler
{
    public $clientUsername;
    public $account;

    public function __construct($parent, $options = array())
    {
        parent::__construct($parent, $options);
        $this->account = new CertumSSLAccount($this->clientUsername);
        $this->helper  = new CertumSSLHelper();
    }

    public function MainHTMLAction($input, $vars = array())
    {
        $account             = new CertumSSLAccount($this->clientUsername);
        $vars['billing_url'] = CertumSSLDriver::config()->billing_url;
        $vars['certs']       = $this->account->getCerts();

        if(isset($_GET['success']) && $_GET['success'] == 'reissued'){
            $vars['success'] = MGLang::T('orderReissued');
        }

        $form = new FormCreator('cert', $this,
            array(
            'customParams' => array(
                'billingUrl' => $vars['billing_url'],
                'cancel' => false,
            )
            )
        );

        if ($this->account->errors && !isset($vars['error'])) {
            $vars['error'] = 'Can\'t connect. Contact With Admin.';
        }

        $form->addField('Text', array(
            'name' => 'order_token'
            , 'require' => true
        ));


        $domainList = array();

        foreach ($account->getDomains() as $domain) {
            $domainList[$domain->domain] = $domain->domain;
        }

        $form->addField('Submit', array(
            'name' => 'action'
            , 'value' => 'addCertificate'
        ));

        $vars['addForm'] = $form->getHTML();

        $cancelForm = new FormCreator('certCancel', $this);
        $form->addField('Hidden', array(
            'name' => 'order_token'
            , 'require' => true
        ));

        return array(
            'template' => 'Main'
            , 'vars' => $vars
        );
    }

    function certCancelHTMLAction()
    {
        $certificate          = new CertumSSLCertificate();
        $certificate->orderId = $_POST["orderID"];
        $certificate->userId  = $this->clientUsername;
        $certificate->cancelCertOrder($_POST["relid"]);

        if (!empty($certificate->errors)) {
            $errorsArray = $certificate->errors;
            echo "<<JSON RESPONSE START>>";
            echo json_encode(array('status' => 'error', 'errors' => json_encode($errorsArray['cancelOrder'])));
            echo "<<JSON RESPONSE END>>";
            die();
        }

        echo "<<JSON RESPONSE START>>";
        echo json_encode(array('status' => 'success', 'msg' => 'Certificate has been cancelled', 'action' => 'CANCEL'));
        echo "<<JSON RESPONSE END>>";
        die();
    }

    function certRevokeHTMLAction()
    {
        if ($_POST["serialNumber"]) {
            $certificate         = new CertumSSLCertificate();
            $certificate->userId = $this->clientUsername;
            $certificate->revokeCertOrder($_POST["serialNumber"], $_POST["relid"]);

            if (!empty($certificate->errors)) {
                $errorsArray = $certificate->errors;
                echo "<<JSON RESPONSE START>>";
                echo json_encode(array('status' => 'error', 'errors' => json_encode($errorsArray['revokeCertificate'])));
                echo "<<JSON RESPONSE END>>";
                die();
            }

            echo "<<JSON RESPONSE START>>";
            echo json_encode(array('status' => 'success', 'action' => 'REVOKING', 'msg' => 'Certificate has been revoked'));
            echo "<<JSON RESPONSE END>>";
            die();
        }

        echo "<<JSON RESPONSE START>>";
        echo json_encode(array('status' => 'error', 'errors' => json_encode(array('Missing Certificate Serial Number'))));
        echo "<<JSON RESPONSE END>>";
        die();
    }

    function getStatementHTMLAction($input, $vars = array())
    {
        $account          = new CertumSSLAccount($this->clientUsername);
        $account->generateToken();
        $apiAccountParams = $account->getAPIAccountParamsByVoucher($input['cert_order_token']);
        if ($apiAccountParams['api_url'] == "" || $apiAccountParams['api_login'] == "" || $apiAccountParams['api_password'] == "") {
            $vars['errors']['general'] = array('You are not allowed to order this product.');
            if (CertumSSLDriver::config()->apilogs_turnon_value == 'on') {
                MGAPILogs::addLog("Client ".$this->clientUsername." are not allowed to order product. Used Token:".$input['cert_order_token'],
                    "Undefined product or API account was not found. Please Check if API Account has correctly setted login,password and api url.");
            }
            return $this->MainHTMLAction($input, $vars);
        }
        $vars['statement'] = $account->getStatement($input['cert_order_token']);

        if ($this->account->errors || $vars['statement'] == false) {
            $vars['errors'] = $this->account->errors;
            return $this->MainHTMLAction($input, $vars);
        }

        $form = new FormCreator('cert', $this, array('customParams' => array('require_legend' => false), 'cancel' => true));

        $form->addField('Hidden', array(
            'name' => 'order_token'
            , 'value' => $_POST["cert_order_token"]
        ));

        $form->addField('Hidden', array(
            'name' => 'statement'
            , 'value' => 1
        ));

        $form->addField('Submit', array(
            'name' => 'action'
            , 'value' => 'addCertificate'
            , 'cancel' => true
        ));

        $vars['form'] = $form->getHTML();


        return array(
            'template' => 'getStatement'
            , 'vars' => $vars
        );
    }

    function certReissueHTMLAction()
    {
        if ($_POST["serialNumber"]) {
            $certificate          = new CertumSSLCertificate();
            $certificate->userId  = $this->clientUsername;
            $certificate->orderId = $_POST["orderID"];
            $certificate->generateCSR(true);

            if (!empty($certificate->errors)) {
                $errorsArray   = $certificate->errors;
                $vars['error'] = $errorsArray['ValidateCSR']['ValidateCSRErrorMessage'];
                return $this->MainHTMLAction($input, $vars);
            }

            $certificate->reissueCertyficate($_POST["serialNumber"], $_POST["relid"]);
            if (!empty($certificate->errors)) {
                $errorsArray = $certificate->errors;
                if (isset($errorsArray['reissueCertificate'])) {
                    foreach ($errorsArray['reissueCertificate'] as $errorArray) {
                        foreach ($errorArray as $errorParam) {
                            $vars['error'] = $errorParam['text'];
                        }
                    }
                } else {
                    foreach ($errorArray as $errortext) {
                        $vars['error'] = $errortext;
                    }
                }
                echo "<<JSON RESPONSE START>>";
                echo json_encode(array('status' => 'error', 'errors' => json_encode(array($vars['error']))));
                echo "<<JSON RESPONSE END>>";
                die();
            }
            $vars['success'] = MGLang::T('orderReissued');
            echo "<<JSON RESPONSE START>>";
            echo json_encode(array('status' => 'success', 'action' => 'REISSUED', 'msg' => $vars['success']));
            echo "<<JSON RESPONSE END>>";
            die();
        }
        $vars['error'] = 'Missing Certyficate Serial Number';
        echo "<<JSON RESPONSE START>>";
        echo json_encode(array('status' => 'error', 'errors' => json_encode(array($vars['error']))));
        echo "<<JSON RESPONSE END>>";
        die();
    }

    function addCertificateHTMLAction($input, $vars = array())
    {

        $fieldErrors  = isset($vars['fieldErrors']) ? $vars['fieldErrors'] : array();
        $certificate  = new CertumSSLCertificate();
        $productCode  = $this->account->getProductCodeFromToken($input['cert_order_token']);
        $SSLProducts  = new SSLProducts();
        $countDomains = $SSLProducts->getDomainsByProductCode($productCode);
        $domainList   = array();

        foreach ($this->account->getDomains() as $domain) {
            $domainList[$domain->domain] = $domain->domain;
        }

        $formParams = array('domainList' => $domainList, "maxDomains" => $countDomains);

        $certificate->orderVoucher = $input['cert_order_token'];

        if ($countDomains > 1) {
            $vars['error'] = MGLang::T('notsupport');
            return $this->MainHTMLAction($input, $vars);
        }


        if (empty($certificate->orderVoucher)) {
            $vars['error'] = MGLang::T('emptytokenError');
            return $this->MainHTMLAction($input, $vars);
        }

        if (!isset($_POST['cert_statement']) && empty($fieldErrors) && empty($vars['error'])) {
            return $this->getStatementHTMLAction($input, $vars);
        }

        $form = new FormCreator('cert', $this, array('cancel' => true));

        $form->addField('Text', array(
            'name' => 'order_token'
            , 'value' => $certificate->orderVoucher
            , 'readonly' => true
        ));

        if ($formParams['maxDomains'] == 1) {
            $form->addField('Select', array(
                'name' => 'domain',
                'options' => $formParams['domainList']
            ));
        } else {
            $form->addField('MultiSelect',
                array(
                'name' => 'domain',
                'options' => $formParams['domainList'],
                'customParams' => array('maxDomains' => $formParams['maxDomains'])
            ));
        }

        $form->addField('Legend', array(
            'name' => 'certdetails'
        ));

        $productsFields = $SSLProducts->productsFields;
        $sslProductType = $SSLProducts->getProductTypeByCode($productCode);
        $productFields  = $productsFields[$sslProductType];
        $fields         = $SSLProducts->fields;

        if (!empty($productFields)) {
            $require = false;
            foreach ($productFields as $productFieldShortcut => $productFieldStatus) {

                $fieldParams = $fields[$productFieldShortcut];

                if ($productFieldStatus == 'required') {
                    $require = true;
                } else {
                    $require = false;
                }

                if ($fieldParams['name'] == 'country') {
                    $form->addField('Select',
                        array(
                        'name' => 'certdetails_'.$fieldParams['name'],
                        'value' => isset($input['cert_certdetails_'.$fieldParams['name']]) ? $input['cert_certdetails_'.$fieldParams['name']] : null,
                        'options' => CertumSSLCertificate::getCountries(),
                        'error' => (!empty($fieldErrors['cert_certdetails_country'])),
                        'require' => $require
                    ));
                } else {
                    $form->addField('Text',
                        array(
                        'name' => 'certdetails_'.$fieldParams['name'],
                        'value' => isset($input['cert_certdetails_'.$fieldParams['name']]) ? $input['cert_certdetails_'.$fieldParams['name']] : null,
                        'error' => (!empty($fieldErorrs['cert_certdetails_'.$fieldParams['name']])),
                        'require' => $require
                    ));
                }
            }
        }

        $form->addField('Legend', array(
            'name' => 'contact'
        ));

        $form->addField('Text',
            array(
            'name' => 'contact_firstname'
            , 'value' => isset($input['cert_contact_firstname']) ? $input['cert_contact_firstname'] : null
            , 'error' => (!empty($fieldErrors['cert_contact_firstname']))
            , 'require' => true
        ));

        $form->addField('Text',
            array(
            'name' => 'contact_lastname'
            , 'value' => isset($input['cert_contact_lastname']) ? $input['cert_contact_lastname'] : null
            , 'error' => (!empty($fieldErrors['cert_contact_lastname']))
            , 'require' => true
        ));

        $form->addField('Text',
            array(
            'name' => 'contact_address'
            , 'value' => isset($input['cert_contact_address']) ? $input['cert_contact_address'] : null
            , 'error' => (!empty($fieldErrors['cert_contact_address']))
            , 'require' => true
        ));

        $form->addField('Select',
            array(
            'name' => 'contact_country',
            'value' => isset($input['cert_contact_country']) ? $input['cert_contact_country'] : null,
            'options' => CertumSSLCertificate::getCountries(),
            'error' => (!empty($fieldErrors['cert_contact_country'])),
            'require' => true
        ));

        $form->addField('Text',
            array(
            'name' => 'contact_city'
            , 'value' => isset($input['cert_contact_city']) ? $input['cert_contact_city'] : null
            , 'error' => (!empty($fieldErrors['cert_contact_city']))
            , 'require' => true
        ));

        $form->addField('Text',
            array(
            'name' => 'contact_email'
            , 'value' => isset($input['cert_contact_email']) ? $input['cert_contact_email'] : null
            , 'error' => (!empty($fieldErrors['cert_contact_email']))
            , 'require' => true
        ));

        $form->addField('Text',
            array(
            'name' => 'contact_phone'
            , 'value' => isset($input['cert_contact_phone']) ? $input['cert_contact_phone'] : null
            , 'error' => (!empty($fieldErrors['cert_contact_phone']))
            , 'require' => true
        ));


        $form->addField('Text',
            array(
            'name' => 'contact_postalcode'
            , 'value' => isset($input['cert_contact_postalcode']) ? $input['cert_contact_postalcode'] : null,
            'error' => (!empty($fieldErrors['cert_contact_postalcode']))
            , 'require' => true
        ));

        $form->addField('Text',
            array(
            'name' => 'contactorg_name',
            'value' => isset($input['cert_contactorg_name']) ? $input['cert_contactorg_name'] : null,
            'error' => (!empty($fieldErrors['cert_contactorg_name'])),
            'require' => false
        ));

        $form->addField('Text',
            array(
            'name' => 'contact_taxIdentificationNumber',
            'value' => isset($input['cert_contact_taxIdentificationNumber']) ? $input['cert_contact_taxIdentificationNumber'] : null,
            'error' => (!empty($fieldErrors['cert_contact_taxIdentificationNumber'])),
            'require' => false
        ));

        $form->addField('Text',
            array(
            'name' => 'org_verifiphonenumber',
            'value' => isset($input['cert_org_verifiphonenumber']) ? $input['cert_org_verifiphonenumber'] : null,
            'error' => (!empty($fieldErrors['cert_org_verifiphonenumber'])),
            'require' => false
        ));

        $form->addField('Submit', array(
            'name' => 'action'
            , 'value' => 'submitCertificate'
            , 'cancel' => true
        ));

        $vars['addForm'] = $form->getHTML();

        return array(
            'template' => 'AddCert'
            , 'vars' => $vars
        );
    }

    function submitCertificateHTMLAction($input, $vars = array())
    {

        $order = new MGCertOrders(array('_tableName' => 'certumssl_orders'));

        if ($order->getOrderByVoucher($input['cert_order_token'])) {
            return $this->MainHTMLAction($input, $vars);
        }

        try {
            $certificate               = new CertumSSLCertificate();
            $certificate->userId       = $this->clientUsername;
            $certificate->orderVoucher = $input['cert_order_token'];
            $voucherdvMethods          = $certificate->getVoucherDVMethod();
            $certificate->setVoucherProductCode();

            $productCode    = $this->account->getProductCodeFromToken($input['cert_order_token']);
            $SSLProducts    = new SSLProducts();
            $sslProductType = $SSLProducts->getProductTypeByCode($productCode);

            if (!is_array($input['cert_domain']) && strpos($sslProductType, "wildcard") !== false) {
                $certificate->domainId = '*.'.$input['cert_domain'];
            } else {
                $certificate->domainId = $input['cert_domain'];
            }
            if (is_array($voucherdvMethods)) {
                foreach ($voucherdvMethods as $voucherdvMethod) {
                    if ($voucherdvMethod == 'DV') {
                        $certificate->DVMethod = 'DV';
                    }
                }
            }

            if (isset($input['cert_approver_email'])) {
                $certificate->approverEmail = $input['cert_approver_email'];
            }

            $certificateInputConditions = $this->helper->certificateInputConditions($input);
            $certificate->orderVoucher  = $input['cert_order_token'];
            $OrganizationName           = null;
            if (isset($input['cert_certdetails_organisation']) && $input['cert_certdetails_organisation'] != '') {
                $OrganizationName = $input['cert_certdetails_organisation'];
            } else {
                $OrganizationName = $input['cert_contact_firstname'].' '.$input['cert_contact_lastname'];
            }

            $certificate->certOrgContacts = $this->helper->certificateInputConditions($input);

            $certificate->certContacts = array(
                'firstName' => $input['cert_contact_firstname'],
                'lastName' => $input['cert_contact_lastname'],
                'address' => $input['cert_contact_address'],
                'city' => $input['cert_contact_city'],
                'country' => $input['cert_contact_country'],
                'email' => $input['cert_contact_email'],
                'phone' => $input['cert_contact_phone'],
                'postalCode' => $input['cert_contact_postalcode'],
                'organisationName' => $input['cert_contactorg_name'],
                'cert_contactorg_name' => $input['cert_contactorg_name'],
                'taxNumber' => $input['cert_contact_taxIdentificationNumber'],
                'verificationPhoneNumber' => $input['cert_org_verifiphonenumber']
            );

            $certificate->certDetails = array(
                'organisation' => $input['cert_certdetails_organisation'],
                'organisation_unit' => $input['cert_certdetails_organisation_unit'],
                'locality' => $input['cert_certdetails_locality'],
                'state_province' => $input['cert_certdetails_state_province'],
                'country' => $input['cert_certdetails_country'],
                'email' => $input['cert_certdetails_email'],
                'serial_number' => $input['cert_certdetails_serial_number'],
                'business_category' => $input['cert_certdetails_business_category'],
                'street_address' => $input['cert_certdetails_street_address'],
                'postal_code' => $input['cert_certdetails_postal_code'],
                'JoILN' => $input['cert_certdetails_JoILN'],
                'JoISoPN' => $input['cert_certdetails_JoISoPN'],
                'JoISoCN' => $input['cert_certdetails_JoISoCN']
            );

            $certificate->taxIdentificationNumber = isset($input['cert_org_taxIdentificationNumber']) ? $input['cert_org_taxIdentificationNumber'] : $input['cert_contact_taxIdentificationNumber'];
            $certificate->verifiPhoneNumber       = isset($input['cert_org_verifiphonenumber']) ? $input['cert_org_verifiphonenumber'] : null;
            
            if ($errors = $certificate->parseCertyficateData($sslProductType)) {
                $vars['fieldErrors'] = $errors;
                return $this->addCertificateHTMLAction($input, $vars);
            } else {

                $certificate->generateCSR();

                if (!empty($certificate->errors) && array_key_exists('ValidateCSR', $certificate->errors)) {
                    $errorsArray   = $certificate->errors;
                    $vars['error'] = $errorsArray['ValidateCSR']['ValidateCSRErrorMessage'];
                    return $this->addCertificateHTMLAction($input, $vars);
                }

                $certificate->placeCertRequest();

                if (!empty($certificate->errors)) {
                    $errorsArray = $certificate->errors;
                    if (isset($errorsArray['quickOrder'][0]) && is_array($errorsArray['quickOrder'][0])) {
                        $vars['error'] = $errorsArray['quickOrder'][0][0]['text'];
                    } else {
                        $vars['error'] = $errorsArray['quickOrder'][0];
                    }

                    if (isset($errorsArray['quickOrder'][0][0]['number']) && $errorsArray['quickOrder'][0][0]['number'] == 1013) {
                        return $this->MainHTMLAction($input, $vars);
                    }

                    return $this->addCertificateHTMLAction($input, $vars);
                }

                $vars['success'] = MGLang::T('orderAddedSuccessfull');
                return $this->MainHTMLAction($input, $vars);
            }
        } catch (\Exception $ex) {
            $vars['error'] = 'ERROR: '.$ex->getMessage();
            return $this->MainHTMLAction($input, $vars);
        }
    }

    function getInfoJSONAction($input, $vars = array())
    {
        if (isset($_POST['data']['orderID'])) {
            $certificate          = new CertumSSLCertificate();
            $certificate->orderId = $_POST['data']['orderID'];
            $certificate->userId  = $this->clientUsername;
            if (isset($_POST['data']['relid'])) {
                $ordersParams = $certificate->getOrderDetails($_POST['data']['relid']);
            } else {
                $ordersParams = $certificate->getOrderDetails();
            }

            foreach ($ordersParams as $orderParam) {
                if ($orderParam['orderdefault']['orderID'] == $_POST['data']['orderID']) {
                    $vars = $orderParam;
                }
            }

            $vars['orderdefault']['voucherCode'] = $_POST['data']['voucherCode'];
            $vars['orderdefault']['domainName']  = $_POST['data']['domainName'];
            $vars['orderdefault']['orderID']     = $_POST['data']['orderID'];
            return json_encode($vars);
        }
    }
}